unit SndConvUnit;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ComCtrls, MMSystem, Grids, ValEdit, ExtCtrls;

const
  { flags in SOLheader.flags, Bit 1 unbekannt, Bit 4 = Stereo }
  solCompressed =   1;
  solUnknown    =   2;
  solIs16bit    =   4;
  solIsSigned   =   8;
  solIsStereo   =  16;

type
   TWAVheader = packed record
       RIFF       : array[0..3] of char; // immer 'RIFF'
       WavLaenge  : longint;             // Lnge ab hier (Datei - 8)
       WAVE       : array[0..3] of char; // immer 'WAVE'
       fmt        : array[0..3] of char; // immer 'fmt_'
       fmtlen     : longint;             //  Lnge des restlichen Headers (16)
       Format     : word;
       Kanal      : word;                // 1= mono, 2=stereo
       Sampling   : longint;             // Rate in Hertz, zB.44100
       ByteProSec : longint;
       ByteProSam : word;                // bei stereo 16 bit = 4 Bytes
       SampleBit  : word;                // 8 / 16 bit Aufnahme
       end;

   PInfoHeader = ^TInfoHeader;
   TInfoHeader = packed record
       data       : array[0..3] of char; // ungleich 'data' !!!
       SamLaenge  : longint;             // Gesamtlnge der RAW-Daten in Bytes
       aLong      : longint;
     end;

   PDataHeader = ^TdataHeader;
   TdataHeader = packed record
       data       : array[0..3] of char; // immer 'data'
       SamLaenge  : longint;             // Gesamtlnge der RAW-Daten in Bytes
     end;

   TSOLheader = packed record
       bID       : byte;  { 0D oder 8D }
       bShift    : byte;  { Lnge des Headers, Audiostart= bShift+2 }
       SOL       : array[0..3] of char; { immer 'SOL'#0 }
       Sampling  : word;
       flags     : byte;
       SamLaenge : longint;
       end;


  TMainForm = class(TForm)
    LadeButton: TButton;
    OpenDialog: TOpenDialog;
    StaBar: TStatusBar;
    DudelButton: TButton;
    Infobox: TGroupBox;
    Valedi: TValueListEditor;
    StopButton: TButton;
    WavStoreButton: TButton;
    WavSaveDialog: TSaveDialog;
    Konv16to8Button: TButton;
    NormButton: TButton;
    SolKonvertButton: TButton;
    SolSaveDialog: TSaveDialog;
    CSaveDialog: TSaveDialog;
    CButton: TButton;
    CRadioFormat: TRadioGroup;
    procedure FormCreate(Sender: TObject);
    procedure LadeButtonClick(Sender: TObject);
    procedure DudelButtonClick(Sender: TObject);
    procedure StopButtonClick(Sender: TObject);
    procedure WavStoreButtonClick(Sender: TObject);
    procedure Konv16to8ButtonClick(Sender: TObject);
    procedure NormButtonClick(Sender: TObject);
    procedure SolKonvertButtonClick(Sender: TObject);
    procedure CButtonClick(Sender: TObject);
  private
    { Private declarations }
    SourceName  : String;
    GesamtLenge : longint;
    SolLenge    : longint;
    isSOL      : boolean;
    procedure  LadeSound;
    procedure  Do_Info;
    procedure  SolKonvert;

  public
    { Public declarations }
  end;

var
  MainForm: TMainForm;
  WavBuf : Array[0..$7FFFFFF] of byte;
  tmpBuf : Array[0..$3FFFFFF] of byte;
  SolHdr : TSOLheader absolute tmpBuf;
  WavHdr : TWavheader absolute WavBuf;
  DataHdr: PdataHeader;
  InfoHdr: PInfoHeader;

const
  Nullwave : TWavheader =
  (    RIFF       : 'RIFF';
       WavLaenge  : 0;
       WAVE       : 'WAVE';
       fmt        : 'fmt ';
       fmtlen     : 16;                 // 16 = ob 16 bit weise
       Format     : 1;
       Kanal      : 1;                  // 1= mono, 2=stereo
       Sampling   : 11025;              // Rate in Hertz, zB.44100
       ByteProSec : 11025;              // Sampling*Kanal*ByteProSam
       ByteProSam : 1;                  // bei stereo 16 bit = 4 Bytes
       SampleBit  : 8;                  // 8 / 16 bit Aufnahme
   );

  Nulldata : TdataHeader =
   (   data       : 'data';
       SamLaenge  : 0;                  // Gesamtlnge der RAW-Daten in Bytes
   );

   NullSol :  TSOLheader =
   (   bID       : $8D;
       bShift    : 12;     { Lnge des Headers, Audiostart= bShift+2 }
       SOL       : 'SOL'#0;
       Sampling  : 0;
       flags     : solCompressed;
       SamLaenge : 0;
    );


implementation

{$R *.dfm}

procedure TMainForm.Do_Info;
var
  S : String;
  Y : integer;
begin
  Infobox.Caption:= SourceName;
  Y:= 1;
  if not isSOL
  then
  begin
  Valedi.Cells[0,Y]:=  'WAV RIFF Id';
  Valedi.Cells[1,Y]:= '"' +  WavHdr.RIFF + '"';
  inc(Y);

  Valedi.Cells[0,Y]:=  'Wavlnge';
  Valedi.Cells[1,Y]:=  IntToStr(WavHdr.WavLaenge);
  inc(Y);

  Valedi.Cells[0,Y]:=  'Formatblock ID';
  Valedi.Cells[1,Y]:=  '"' + WavHdr.fmt + '"';
  inc(Y);

  Valedi.Cells[0,Y]:=  'Formatblock Lnge';
  Valedi.Cells[1,Y]:=  IntToStr(WavHdr.fmtlen);
  inc(Y);

  Valedi.Cells[0,Y]:=  'Audioformat';
  case WavHdr.Format of
    $001: S:= 'PCM (WAV)';
    $002: S:= 'MS ADPCM';
    $003: S:= 'IEEE FLOAT';
    $005: S:= 'IBM CVSD';
    $006: S:= 'ALAW';
    $007: S:= 'MULAW';
    $010: S:= 'OKI ADPCM';
    $011: S:= 'DVI/IMA ADPCM';
    $012: S:= 'MEDIASPACE ADPCM';
    $013: S:= 'SIERRA ADPCM';
    $014: S:= 'G723 ADPCM';
    $015: S:= 'DIGISTD';
    $016: S:= 'DIGIFIX';
    $017: S:= 'DIALOGIC OKI ADPCM';
    $020: S:= 'YAMAHA ADPCM';
    $021: S:= 'SONARC';
    $022: S:= 'DSPGROUP TRUESPEECH';
    $023: S:= 'ECHOSC1';
    $024: S:= 'AUDIOFILE AF36';
    $025: S:= 'APTX';
    $026: S:= 'AUDIOFILE AF10';
    $030: S:= 'DOLBY AC2';
    $031: S:= 'GSM610';
    $033: S:= 'ANTEX ADPCME';
    $034: S:= 'CONTROL RES VQLPC';
    $035: S:= 'CONTROL RES VQLPC';
    $036: S:= 'DIGIADPCM';
    $037: S:= 'CONTROL RES CR10';
    $038: S:= 'NMS VBXADPCM';
    $039: S:= 'CS IMAADPCM (Roland RDAC)';
    $040: S:= 'G721 ADPCM';
    $050: S:= 'MPEG-1 Layer I, II';
    $055: S:= 'MPEG-1 Layer III (MP3)';
    $069: S:= 'Xbox ADPCM';
    $200: S:= 'CREATIVE ADPCM';
    $202: S:= 'CREATIVE FASTSPEECH8';
    $203: S:= 'CREATIVE FASTSPEECH10';
    $300: S:= 'FM TOWNS SND';
    $1000: S:= 'OLIGSM';
    $1001: S:= 'OLIADPCM';
    $1002: S:= 'OLICELP';
    $1003: S:= 'OLISBC';
    $1004: S:= 'OLIOPR';
  else S:= 'sonstig, kein WAV';
  end;
  Valedi.Cells[1,Y]:=  IntToStr(WavHdr.Format)+' '+S;
  inc(Y);

  Valedi.Cells[0,Y]:=  'Kanal-Anzahl';
  Valedi.Cells[1,Y]:=  IntToStr(WavHdr.Kanal);
  inc(Y);

  Valedi.Cells[0,Y]:=  'Samples/Sec';
  Valedi.Cells[1,Y]:=  IntToStr(WavHdr.Sampling);
  inc(Y);

  Valedi.Cells[0,Y]:=  'Bytes/Sec';
  Valedi.Cells[1,Y]:=  IntToStr(WavHdr.ByteProSec);
  inc(Y);

  Valedi.Cells[0,Y]:=  'Bytes/Sample';
  Valedi.Cells[1,Y]:=  IntToStr(WavHdr.ByteProSam);
  inc(Y);

  Valedi.Cells[0,Y]:=  'Bits/Sample';
  Valedi.Cells[1,Y]:= IntToStr(WavHdr.SampleBit);
  inc(Y);

  if InfoHdr<>nil
  then
  begin
    Valedi.Cells[0,Y]:=  'InfoHeader Id';
    Valedi.Cells[1,Y]:= '"' + InfoHdr^.data + '"';
    inc(Y);

    Valedi.Cells[0,Y]:=  'InfoHeader Len';
    Valedi.Cells[1,Y]:= IntToStr(InfoHdr^.SamLaenge);
    inc(Y);

    Valedi.Cells[0,Y]:=  'InfoHeader aLong';
//    Valedi.Cells[1,Y]:= '0x' + IntToHex(InfoHdr^.aLong,8);
    Valedi.Cells[1,Y]:=  IntToStr(InfoHdr^.aLong);
    inc(Y);
  end;

  if DataHdr<>nil
  then
  begin
    Valedi.Cells[0,Y]:=  'DataHeader Id';
    Valedi.Cells[1,Y]:= '"' + DataHdr^.data + '"';
    inc(Y);

    Valedi.Cells[0,Y]:=  'SamLnge';
    Valedi.Cells[1,Y]:= IntToStr(DataHdr^.SamLaenge);
    inc(Y);
  end;
  while Y<20 do
  begin
    Valedi.Cells[0,Y]:= '';
    Valedi.Cells[1,Y]:= '';
    inc(Y);
  end;
  end
  else
  begin

   Valedi.Cells[0,Y]:= 'SOL Id Byte';
   Valedi.Cells[1,Y]:= '0x'+IntToHex(SolHdr.bID,2);
   inc(Y);

   Valedi.Cells[0,Y]:= 'Shift';
   Valedi.Cells[1,Y]:= IntToStr(SolHdr.bShift);
   inc(Y);

   Valedi.Cells[0,Y]:= 'Ident';
   Valedi.Cells[1,Y]:= SolHdr.SOL;
   inc(Y);

   Valedi.Cells[0,Y]:= 'Sampling';
   Valedi.Cells[1,Y]:= IntToStr(SolHdr.Sampling);
   inc(Y);

   Valedi.Cells[0,Y]:= 'Flags';
   Valedi.Cells[1,Y]:= '0x'+IntToHex(SolHdr.flags,2);
   inc(Y);

   Valedi.Cells[0,Y]:= 'SamLnge';
   Valedi.Cells[1,Y]:= IntToStr(SolHdr.SamLaenge);
   inc(Y);

   S:= 'wav';
   if (SolHdr.flags and solCompressed) <> 0 then S:= 'compr, ';
   if (SolHdr.flags and solUnknown)    <> 0 then S:= S + '??, ';
   if (SolHdr.flags and solIs16bit)    <> 0
    then S:= S + '16bit, ' else S:= S + '8bit, ';
   if (SolHdr.flags and solIsSigned)   <> 0
    then S:= S + 'signed, ' else S:= S + 'unsigned, ';
   if (SolHdr.flags and solIsStereo)    <> 0
    then S:= S + 'stereo' else S:= S + 'mono';
   Valedi.Cells[0,Y]:= 'verbose';
   Valedi.Cells[1,Y]:= S;
   inc(Y);

  while Y<20 do
  begin
    Valedi.Cells[0,Y]:= '';
    Valedi.Cells[1,Y]:= '';
    inc(Y);
  end;
  end;
end;

procedure TMainForm.FormCreate(Sender: TObject);
begin
  FillChar(WavBuf,sizeof(WavBuf),0);
  FillChar(tmpBuf,sizeof(tmpBuf),0);
  SourceName:= '';
  GesamtLenge:= 0;
  StaBar.Panels[0].Text:= '';
  StaBar.Panels[1].Text:= ''
end;


procedure TMainForm.LadeButtonClick(Sender: TObject);
var
  b : boolean;
begin
  if OpenDialog.Execute
  then begin
         SourceName:= OpenDialog.FileName;
         LadeSound;
         b:= WavHdr.Format = 1;
         SolKonvertButton.Enabled:= b;
         CButton.Enabled:= b;
         NormButton.Enabled:= b;
         Konv16to8Button.Enabled:= b;
       end;
end;


procedure TMainForm.DudelButtonClick(Sender: TObject);
begin
  PlaySound(@WavBuf,0,SND_MEMORY or SND_ASYNC);
end;


procedure TMainForm.StopButtonClick(Sender: TObject);
begin
  PlaySound(nil,0,0);
end;


const
SOLTable3bit : array[0..7] of byte = ( 0, 1, 2, 3, 6, 10, 15, 21 );
SOLTable7bit : array[0..127] of word =
  (   $0,    $8,   $10,   $20,   $30,   $40,   $50,   $60,
     $70,   $80,   $90,   $A0,   $B0,   $C0,   $D0,   $E0,
     $F0,  $100,  $110,  $120,  $130,  $140,  $150,  $160,
    $170,  $180,  $190,  $1A0,  $1B0,  $1C0,  $1D0,  $1E0,
    $1F0,  $200,  $208,  $210,  $218,  $220,  $228,  $230,
    $238,  $240,  $248,  $250,  $258,  $260,  $268,  $270,
    $278,  $280,  $288,  $290,  $298,  $2A0,  $2A8,  $2B0,
    $2B8,  $2C0,  $2C8,  $2D0,  $2D8,  $2E0,  $2E8,  $2F0,
    $2F8,  $300,  $308,  $310,  $318,  $320,  $328,  $330,
    $338,  $340,  $348,  $350,  $358,  $360,  $368,  $370,
    $378,  $380,  $388,  $390,  $398,  $3A0,  $3A8,  $3B0,
    $3B8,  $3C0,  $3C8,  $3D0,  $3D8,  $3E0,  $3E8,  $3F0,
    $3F8,  $400,  $440,  $480,  $4C0,  $500,  $540,  $580,
    $5C0,  $600,  $640,  $680,  $6C0,  $700,  $740,  $780,
    $7C0,  $800,  $900,  $A00,  $B00,  $C00,  $D00,  $E00,
    $F00, $1000, $1400, $1800, $1C00, $2000, $3000, $4000 );


Adpcm_idx1 : array[0..31] of integer =
   ( -1, -1, -1, -1, -1, -1, -1, -1,  1,  2,  4,  6,  8, 10, 13, 16,
     -1, -1, -1, -1, -1, -1, -1, -1,  1,  2,  4,  6,  8, 10, 13, 16);

Adpcm_idx2 : array[0..15] of integer =
   ( -1, -1, -1, -1,  2,  4,  6,  8,
     -1, -1, -1, -1,  2,  4,  6,  8 );

Adpcm_idx3 : array[0..7] of integer =
   ( -1, -1,  1,  2, -1, -1,  1,  2 );



Adpcm_Tbl : array[0..88] of longint =

({ 0.. 9}      7,     8,     9,    10,    11,    12,     13,    14,    16,    17,
 {10..19}     19,    21,    23,    25,    28,    31,     34,    37,    41,    45,
 {20..29}     50,    55,    60,    66,    73,    80,     88,    97,   107,   118,
 {30..39}    130,   143,   157,   173,   190,   209,    230,   253,   279,   307,
 {40..49}    337,   371,   408,   449,   494,   544,    598,   658,   724,   796,
 {50..59}    876,   963,  1060,  1166,  1282,  1411,   1552,  1707,  1878,  2066,
 {60..69}   2272,  2499,  2749,  3024,  3327,  3660,   4026,  4428,  4871,  5358,
 {70..79}   5894,  6484,  7132,  7845,  8630,  9493,  10442, 11487, 12635, 13899,
 {80..88}  15289, 16818, 18500, 20350, 22385, 24623,  27086, 29794, 32767 );


procedure  TMainForm.LadeSound;
var
  F : File;
  T : Textfile;
  step,
  rp, wp,
  li, re,
  r, rr,
  L    : longint;
  idx  : longint;
  b, q : byte;


  procedure Deco_ADPCM (nibble: byte);
  var
    alt,
    neu,
    idiff : longint;
    diff  : single;
    S : String;
    altidx : integer;

  begin
    Str(r:5,S);

    alt:= li;
    nibble:= nibble and 15;

    altidx:= idx;
    step:= Adpcm_Tbl[idx];
    diff:= nibble and 7;

    if (nibble and 8) <> 0
    then  S:= S + ': -' + chr((nibble and 7)+48)
    else  S:= S + ': +' + chr((nibble and 7)+48);

    diff:= (diff + 0.5) * step;
    diff:= diff / 4.0;


    if (nibble and 8) <> 0 then diff:= -diff;

    idiff:= trunc(diff);
    neu:= alt + idiff;
    li:= neu;
    if li >  32737  then li:=  32737;
    if li < -32738  then li:= -32738;

    WavBuf[wp]:= (li shr 0) and $FF; inc(wp);
    WavBuf[wp]:= (li shr 8) and $FF; inc(wp);

    idx:= idx + Adpcm_idx2[nibble];
    if idx < 0  then idx:= 0;
    if idx > 88 then idx:= 88;
    inc(r);
    inc(DataHdr^.SamLaenge,2);
    writeln(T, S, ' idx: ',altidx:2, alt:7,'-> ',diff:7:1,' -> ',neu:7);
  end;


begin
  InfoHdr:= nil;
  DataHdr:= nil;
  isSOL:= false;
  AssignFile(F,SourceName);
  reset(F,1);
  BlockRead(F, WavBuf, 256, r);

  if r<>256            // pauschal
  then begin
         beep;
         CloseFile(F);
         exit;
       end;
  if (WavHdr.RIFF = 'RIFF') and
     (WavHdr.WAVE = 'WAVE')
  then begin
         L:= FileSize(F);
         if L > sizeof(tmpBuf) then  L:= sizeof(tmpBuf);
         Seek(F,0);
         BlockRead(F,tmpBuf, L);
         closefile(F);
         StaBar.Panels[0].Text:= IntToStr(L)+' Bytes gelesen';

         r:= 20 + WavHdr.fmtlen;
         DataHdr:= @WavBuf[r];
         if DataHdr^.data <> 'data'
         then begin
                InfoHdr:= @WavBuf[r];
                DataHdr:= @WavBuf[r+sizeof(TInfoHeader)];
                inc(r,sizeof(TInfoHeader));
              end;
         inc(r,sizeof(TDataHeader));
         rp:= r;


         wp:= sizeof(WavHdr)+ sizeof(TdataHeader);
         Do_Info;


         { Vorsicht: das Dekodieren von ADPCM hat noch eine Macke.
           Inhalt des Info-"fact"-Headers ist unbekannt.       }

         if WavHdr.Format = 17      { DVI ADPCM }
         then
         begin
           assignfile(T,'adpcm.log');
           rewrite(T);


           rr:= L;
           if InfoHdr <> nil
           then rr:= rp + (InfoHdr^.aLong div 2);
           WavHdr.Format:= 1;
           WavHdr.fmtlen:= 16;
           WavHdr.SampleBit:= 16;
           WavHdr.ByteProSam:= 2;
           WavHdr.ByteProSec:= 2 * WavHdr.Sampling;
           DataHdr:= @WavBuf[sizeof(WavHdr)];
           DataHdr^.Data:= 'data';
           DataHdr^.SamLaenge:= 0;

           r:= 1;                   { Zhler }
           li:= 0;                  { Startwert fr Samples  }
           idx:= 0;                 { Startwert fr Index    }
           step:= 0;
           while rp < rr do
           begin
             Deco_ADPCM(tmpBuf[rp]);
             Deco_ADPCM(tmpBuf[rp] shr 4);
             inc(rp);
           end;
           L:= wp;
           WavHdr.WavLaenge:= L - 8;

           writeln(T,'..gesamt ',L,' Bytes');
           closefile(T);
         end
         else
         begin
           move(tmpBuf,WavBuf,L);
         end;
         GesamtLenge:= L;
         exit;
       end;

  Seek(F,0);
  L:= FileSize(F);
  if L > sizeof(tmpBuf) then  L:= sizeof(tmpBuf);
  BlockRead(F,tmpBuf, L);
  closefile(F);

  if SolHdr.SOL <> 'SOL'
  then begin
         beep;
         exit;
       end;

  isSOL:= true;
  StaBar.Panels[0].Text:= IntToStr(L)+' Bytes gelesen';
  GesamtLenge:= L;
  DataHdr:= nil;
  InfoHdr:= nil;
  Do_Info;

  WavHdr:= Nullwave;
  DataHdr:= @WavBuf[sizeof(WavHdr)];
  DataHdr^.data:= 'data';

  if (SolHdr.flags and solIs16bit) <> 0
  then
  begin
   { SOL ist 16 Bit }
  WavHdr.Sampling  := SolHdr.Sampling;
  Wavhdr.ByteProSec:= SolHdr.Sampling*4;
  WavHdr.SampleBit := 16;
  WavHdr.Kanal     := 2;
  WavHdr.ByteProSam:= 4;
  DataHdr^.SamLaenge := 2 * SolHdr.SamLaenge;
  WavHdr.WavLaenge := DataHdr^.SamLaenge+44-8;
  rp:= sizeof(TSolHeader);
  wp:= sizeof(TWavHeader)+sizeof(TDataHeader);
  li:= 0; re:= 0;               // Startwert
  WavBuf[wp]:= 0; inc(wp);
  WavBuf[wp]:= 0; inc(wp);
  while rp < L do
  begin
    b:= tmpBuf[rp];
    if (b and 128) <> 0
    then begin
           li:= li - SOLTable7bit[b and 127];
         end
    else begin
           li:= li + SOLTable7bit[b and 127];
         end;
    if li<-32767 then li:= -32767;
    if li>32767  then li:= 32767;
    WavBuf[wp]:= li and 255; inc(wp);
    WavBuf[wp]:= (li shr 8) and 255; inc(wp);
    inc(rp);

  if (SolHdr.flags and solIsStereo) <> 0
  then
  begin
    b:= tmpBuf[rp];
    if (b and 128) <> 0
    then begin
           re:= re - SOLTable7bit[b and 127];
         end
    else begin
           re:= re + SOLTable7bit[b and 127];
         end;
    if re<-32767 then re:= -32767;
    if re>32767  then re:= 32767;

    WavBuf[wp]:= re and 255; inc(wp);
    WavBuf[wp]:= (re shr 8) and 255; inc(wp);
    inc(rp);
   end;
  end;
   GesamtLenge:= wp;
   WavHdr.WavLaenge:= wp;
   while wp < sizeof(WavBuf) do
   begin
     WavBuf[wp]:= 0;
     inc(wp);
   end;

   exit;
  end;

  { SOL ist 8 Bit }
  WavHdr:= Nullwave;
  WavHdr.Sampling  := SolHdr.Sampling;
  Wavhdr.ByteProSec:= SolHdr.Sampling;
  WavHdr.SampleBit := 8;
  DataHdr^.SamLaenge := 2 * SolHdr.SamLaenge;
  WavHdr.WavLaenge := DataHdr^.SamLaenge+44-8;
  rp:= sizeof(TSolHeader);
  wp:= sizeof(TWavHeader)+sizeof(TDataHeader);
  li:= 128;               // Startwert
  WavBuf[wp]:= li; inc(wp);
  while rp < L do
  begin
    b:= (tmpBuf[rp] shr 4) and 15;
    if (b and 8) <> 0
    then begin
           q:= 15 - b;
           li:= li - SOLTable3bit[q];
         end
    else begin
           q:= b and 7;
           li:= li + SOLTable3bit[q];
         end;
    if li<0 then li:= 0;
    if li>255 then li:= 255;
    WavBuf[wp]:= li; inc(wp);

    b:= tmpBuf[rp] and 15;
    if (b and 8) <> 0
    then begin
           q:= 15 - b;
           li:= li - SOLTable3bit[q];
         end
    else begin
           q:= b and 7;
           li:= li + SOLTable3bit[q];
         end;
    if li<0 then li:= 0;
    if li>255 then li:= 255;
    WavBuf[wp]:= li; inc(wp);
    inc(rp);
   end;
   GesamtLenge:= wp;
   WavHdr.WavLaenge:= wp;
   while wp < sizeof(WavBuf) do
   begin
     WavBuf[wp]:= 128;
     inc(wp);
   end;
end;



procedure TMainForm.WavStoreButtonClick(Sender: TObject);
var
  F : File;
  L : longint;
begin
  if WavSaveDialog.Execute
  then begin
         L:= WavHdr.WavLaenge + 8;
         AssignFile(F, WavSaveDialog.FileName);
         rewrite(F,1);
         BlockWrite(F,WavBuf,L);
         closefile(F);
       end;
end;

procedure TMainForm.Konv16to8ButtonClick(Sender: TObject);
var
  li, re,
  i, j : longint;
  P : ^smallint;
begin
  if WavHdr.SampleBit <> 16 then exit;
  i:= sizeof(TWavHeader)+sizeof(TdataHeader);
  j:= sizeof(TWavHeader)+sizeof(TdataHeader);
  DataHdr:= @WavBuf[sizeof(TWavHeader)];
  DataHdr^.data:= 'data';
  while i < GesamtLenge do
  begin
    P:= @WavBuf[i];
    li:= P^;
    inc(i,2);
    P:= @WavBuf[i];
    re:= P^;
    inc(i,2);

    if WavHdr.Kanal = 2
    then
    begin
      li:= (li + re) div 512;
      WavBuf[j]:= li + 128;
      inc(j);
    end
    else
    begin
     li:= li div 256;
     re:= re div 256;
     WavBuf[j]:= li + 128;
     WavBuf[j+1]:= re + 128;
     inc(j,2);
    end;
  end;
  GesamtLenge:= j;
  WavHdr.SampleBit:= 8;
  DataHdr^.SamLaenge:= DataHdr^.SamLaenge div 2;
  if WavHdr.Kanal = 2
  then  begin
          WavHdr.Kanal:= 1;
          DataHdr^.SamLaenge:= DataHdr^.SamLaenge div 2;
        end;
  WavHdr.WavLaenge:= DataHdr^.SamLaenge + sizeof(TWavHeader)+sizeof(TdataHeader);
end;

procedure TMainForm.NormButtonClick(Sender: TObject);
var
  i : longint;
  li, re,
  lmax, rmax : longint;
  fl, fr : single;
  P : ^smallint;
begin
  i:= sizeof(TWavHeader);
  if WavHdr.SampleBit = 16
  then
  begin
       { es ist ein 16 Bit WAV  }
    lmax:= 0;
    rmax:= 0;
    while i < GesamtLenge do
    begin
      P:= @WavBuf[i];
      li:= P^;
      if abs(li) > lmax then lmax:= abs(li);
      inc(i,2);

      P:= @WavBuf[i];
      re:= P^;
      if abs(re) > rmax then rmax:= abs(re);
      inc(i,2);
    end;
    if WavHdr.Kanal = 1                 { mono }
    then begin
           if lmax > rmax
           then rmax:= lmax
           else lmax:= rmax;
         end;
    i:= sizeof(TWavHeader);
    fl:=  32500.0 / lmax;          { nicht ganz Vollausschlag }
    fr:=  32500.0 / rmax;
    StaBar.Panels[1].Text:= 'L '+IntToStr(lmax)+' R '+IntToStr(rmax);
    while i < GesamtLenge do
    begin
      P:= @WavBuf[i];
      li:= P^;
      li:= round(li * fl);
      if li > 32767 then li:= 32767;
      if li < -32767 then li:= -32767;
      P^:= li;
      inc(i,2);

      P:= @WavBuf[i];
      re:= P^;
      re:= round(re * fr);
      if re > 32767 then re:= 32767;
      if re < -32767 then re:= -32767;
      P^:= re;
      inc(i,2);
    end;
    exit;
  end;

  lmax:= 0;
  while i < GesamtLenge do
  begin
    li:= WavBuf[i];
    li:= li - 128;
    if abs(li) > lmax then lmax:= abs(li);
    inc(i);
  end;

  fl:= 126 / lmax;          { nicht ganz Vollausschlag }
  i:= sizeof(TWavHeader);
  while i < GesamtLenge do
  begin
    li:= WavBuf[i];
    li:= li - 128;
    li:= round(li * fl) + 128;
    if li > 255 then li:= 255;
    if li < 0 then li:= 0;
    WavBuf[i] := li;
    inc(i);
  end;

end;

const
SOLTable4bit : array[0..15] of integer =
  ( 0,   1,   2,  3,  6, 10, 15, 21,
  -21, -15, -10, -6, -3, -2, -1,  0 );


{ erzeugt aus Wav (8 Bit, mono) eine SOL-kodierte Audiodatei }
procedure  TMainForm.SolKonvert;
var
  i, j : longint;
  oldsample,
  newsample : longint;
  B      : byte;
  diff   : integer;
  rest   : integer;

   function MakeNibble: byte;
   var
     k, l, m  : integer;
     delta    : integer;
   begin
     { wir knnen berhnge haben, die aus nicht ganz getroffenen
       vorherigen Samples herrhren }
     diff:= newsample - oldsample + rest;

     { den nchstbesten Sttzpunkt suchen,
       also wer am nchsten zu diff liegt }
     delta:= 100000;
     k:= 15;
     for l:= 0 to 15 do
     begin
       m:= SOLTable4bit[l];
       if abs(diff - m) < abs(diff - delta)
       then begin
              delta:= m;
              k:= l;
            end;
     end;
     rest:= diff - delta;
     result:= k;
     oldsample:= newsample;
     exit;
   end;


begin
  if (WavHdr.SampleBit <> 8) or
     (WavHdr.Kanal <> 1)
  then begin
         beep;
         exit;
       end;
  SolHdr:= NullSol;
  SolHdr.Sampling:= WavHdr.Sampling;
  i:= sizeof(TWavHeader)+sizeof(TDataHeader);
  j:= sizeof(TSolHeader);
  SolLenge:= j;
  oldsample:= 0;
  diff:= 0;

  while i < GesamtLenge do
  begin
    newsample:= WavBuf[i] - 128;
    B:= MakeNibble shl 4;
    inc(i);

    newsample:= WavBuf[i] - 128;
    B:= B or MakeNibble;
    inc(i);

    tmpBuf[j]:= B;
    inc(j);
  end;
  SolLenge:= j;
  SolHdr.SamLaenge:= j - sizeof(TSolHeader);
end;




procedure TMainForm.SolKonvertButtonClick(Sender: TObject);
var
  F : File;
begin
  SolKonvert;
  if SolSaveDialog.Execute
  then begin
         AssignFile(F, SolSaveDialog.FileName);
         rewrite(F,1);
         BlockWrite(F,tmpBuf,SolLenge);
         closefile(F);
       end;
end;

procedure TMainForm.CButtonClick(Sender: TObject);
var
  F : TextFile;
  a,b,c,d : byte;
  q,r : byte;
  i,L : longint;
  S   : String;
  mach_wav : boolean;
begin
  mach_wav:= CRadioFormat.ItemIndex <> 0;
  SolKonvert;
  S:= ExtractFileName(SourceName);
  i:= pos('.',S);
  if i<>0 then S:= copy(S,1,i-1);
  CSaveDialog.FileName:= S + '.inc';

  L:= SolHdr.SamLaenge * 2;
  a:= L and $FF;
  b:= (L shr 8) and $FF;
  c:= (L shr 16) and $FF;
  d:= (L shr 24) and $FF;
  q:= SolHdr.Sampling and $FF;
  r:= (SolHdr.Sampling shr 8) and $FF;

  if CSaveDialog.Execute
  then begin
         AssignFile(F, CSaveDialog.FileName);
         rewrite(F);
         writeln(F,'/* Audiofile aus ', SourceName, ' */');
         writeln(F,'const char ', S, '[] =');
         writeln(F,'{ ',d,', ',c,', ',b,', ',a,', /* ',L,' Samples bigendian */');
         writeln(F, '0, 0, ',r,', ',q,',   /* ',SolHdr.Sampling,' Samplefrequenz bigendian */');
         if mach_wav
         then writeln(F,'0, /* 0 = isWAV */')
         else writeln(F,'1, /* 1 = isSOL */');

         if mach_wav
         then
         begin
           L:= sizeof(WavHdr);
           if InfoHdr<>nil then inc(L,sizeof(TInfoHeader));
           i:= 0;
           while L < (WavHdr.WavLaenge+8) do
           begin
             write(F, WavBuf[L],', ');
             inc(L);
             inc(i);
             if i > 10
             then begin
                    i:= 0;
                    writeln(F);
                  end
           end;
         end
         else
         begin
           L:= sizeof(SolHdr);
           i:= 0;
           while L < SolLenge do
           begin
             write(F, tmpBuf[L],', ');
             inc(L);
             inc(i);
             if i > 10
             then begin
                    i:= 0;
                    writeln(F);
                  end
           end;
         end;
         writeln(F,'};');
         closefile(F);
       end;
end;

end.
